
# q-expansion of newform 360.2.k.e, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 360
weight = 2
poly_data = [1, 0, -1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 1, 0], 1], [[0, 0, 0, 1], 1], [[0, 1, -1, 0], 1]]

hecke_ring_character_values = [[271, [1, 0, 0, 0]], [181, [-1, 0, 0, 0]], [281, [1, 0, 0, 0]], [217, [1, 0, 0, 0]]]
aps_data = [[0, 1, -1, 0], [0, 0, 0, 0], [0, 0, 1, 0], [-1, 1, -1, 1], [0, 0, 2, 0], [2, -2, 0, 2], [0, -2, 2, -2], [2, -2, 4, 2], [1, 3, -3, 3], [4, -4, 0, 4], [-2, -2, 2, -2], [0, 0, -2, 0], [2, -2, 2, -2], [-1, 1, -7, -1], [-5, 1, -1, 1], [2, -2, -8, 2], [-2, 2, 4, -2], [4, -4, -2, 4], [-1, 1, 9, -1], [-2, -2, 2, -2], [4, 2, -2, 2], [-8, 4, -4, 4], [-1, 1, -3, -1], [-2, -4, 4, -4], [-4, -6, 6, -6], [4, -4, 4, 4], [7, 5, -5, 5], [1, -1, -1, 1], [-4, 4, 10, -4], [6, 4, -4, 4], [15, 1, -1, 1], [-8, 8, 6, -8], [2, -4, 4, -4], [-2, 2, -4, -2], [-8, 8, -6, -8], [-2, -6, 6, -6], [-4, 4, -10, -4], [3, -3, 5, 3], [15, -3, 3, -3], [0, 0, -2, 0], [2, -2, 12, 2], [0, 0, 16, 0], [2, -10, 10, -10], [4, -2, 2, -2], [-2, 2, 16, -2], [12, -8, 8, -8], [-12, 12, 6, -12], [-11, 3, -3, 3], [11, -11, 9, 11], [0, 0, -4, 0], [-12, 10, -10, 10], [20, 0, 0, 0], [6, 6, -6, 6], [4, -4, -18, 4], [2, 0, 0, 0], [3, 5, -5, 5], [-4, 4, 2, -4], [-2, -10, 10, -10], [0, 0, 2, 0], [-14, 2, -2, 2], [5, -5, -1, 5], [-8, 8, 2, -8], [15, -15, 1, 15], [-14, -10, 10, -10], [-18, 8, -8, 8], [-2, 2, -12, -2], [0, 0, -14, 0], [-6, -8, 8, -8], [-5, 5, -7, -5], [0, 0, -28, 0], [-6, -4, 4, -4], [-12, -4, 4, -4], [15, -7, 7, -7], [16, -16, 2, 16], [14, -14, -12, 14], [-9, -7, 7, -7], [12, -12, 14, 12], [-14, 14, 8, -14], [-6, 8, -8, 8], [6, -10, 10, -10], [6, -6, -8, 6], [8, -8, 14, 8], [18, 2, -2, 2], [-16, -2, 2, -2], [20, 12, -12, 12], [15, -15, 5, 15], [6, 10, -10, 10], [-6, -12, 12, -12], [-4, 4, 4, -4], [-13, 1, -1, 1], [-5, 5, 17, -5], [8, -8, 8, -8], [-9, 9, -9, 9], [-4, 4, 10, -4], [-2, 2, 28, -2], [9, -5, 5, -5], [-8, 8, -12, -8], [30, 8, -8, 8], [-3, 3, -17, -3], [4, -4, -20, 4], [1, -1, 35, 1], [12, -12, 6, 12], [-11, 11, 3, -11], [10, 2, -2, 2], [-12, 12, -14, -12], [-2, 24, -24, 24], [3, -3, 9, 3], [2, -20, 20, -20], [0, 20, -20, 20], [22, -2, 2, -2], [5, 15, -15, 15], [-6, 6, 16, -6], [24, -2, 2, -2], [-10, 10, -16, -10], [-6, -10, 10, -10], [10, -6, 6, -6], [-7, 7, 27, -7], [-15, -1, 1, -1], [14, -14, 12, 14], [-10, 10, 0, -10], [-8, 8, -22, -8], [16, -2, 2, -2], [-6, 6, -28, -6], [-15, 15, -9, -15], [0, 0, 18, 0], [-4, 4, -26, -4], [12, -12, 8, 12], [-12, 8, -8, 8], [-5, -11, 11, -11], [-4, 4, -42, -4], [-10, 10, 12, -10], [15, -15, 15, -15], [2, 14, -14, 14], [-4, 4, -26, -4], [-22, 16, -16, 16], [6, 4, -4, 4], [-6, 6, 12, -6], [-9, 9, 1, -9], [22, -22, -12, 22], [-10, -8, 8, -8], [4, -4, -22, 4], [12, -12, -14, 12], [3, -7, 7, -7], [7, -7, -11, 7], [-4, 4, 22, -4], [-4, -12, 12, -12], [-6, 6, -32, -6], [6, 8, -8, 8], [-14, 14, 4, -14], [-39, -5, 5, -5], [16, -16, 26, 16], [-6, -2, 2, -2], [19, -19, 5, 19], [-19, 19, -19, 19], [7, -7, 17, 7], [18, 18, -18, 18], [-32, -4, 4, -4], [14, 2, -2, 2], [-20, 14, -14, 14], [0, 0, 32, 0], [-7, 7, -9, -7], [38, -12, 12, -12], [25, -5, 5, -5], [4, -4, -30, 4], [28, -2, 2, -2], [47, 1, -1, 1], [38, 2, -2, 2], [18, -18, 20, 18]]
